#!/usr/bin/env python
"""
    AwesomeTkinter, a new tkinter widgets design using custom styles and images

    :copyright: (c) 2020 by Mahmoud Elshahat.

    module description:
        this is the main module which get executed when running command python -m awesometkinter
"""


import sys
import os

if __package__ is None and not hasattr(sys, 'frozen'):
    # direct call of __main__.py
    path = os.path.realpath(os.path.abspath(__file__))
    sys.path.insert(0, os.path.dirname(os.path.dirname(path)))

    __package__ = 'awesometkinter'

from .__init__ import main


if __name__ == '__main__':
    main() 



"""
    AwesomeTkinter, a new tkinter widgets design using custom styles and images

    :copyright: (c) 2020 by Mahmoud Elshahat.

"""

from .utils import *
from .config import *
from .images import *


class Button3d(ttk.Button):
    """create a button with 3d background color and shadow"""
    styles = []

    def __init__(self, parent, bg=None, fg=None, **options):
        """initialize

        Args:
            parent: tkinter container widget, i.e. root or another frame
            bg (str): button color
            fg (str): text color
        """
        self.bg = bg or DEFAULT_COLOR
        parent_color = get_widget_attribute(parent, 'background') or DEFAULT_COLOR

        # initialize super class
        ttk.Button.__init__(self, parent, **options)

        # create unique style name based on frame color
        button_style = f'Button3d_{len(Button3d.styles)}_.TButton'

        # create style
        if button_style not in Button3d.styles:
            self.create_images()

            # create elements
            s = ttk.Style()
            element_name = f'{button_style}_element'
            s.element_create(element_name, 'image', self.img, ('pressed', self.pressed_img), border=12, sticky="nsew")
            s.layout(button_style,
                     [('Button.border', {'sticky': 'nswe', 'border': '1', 'children':
                         [(element_name, {'sticky': 'nswe', 'children':
                             [('Button.padding', {'sticky': 'nswe', 'children':
                                 [('Button.label', {'sticky': 'nswe'})]})]})]})])
            s.map(button_style, background=[('', parent_color)], foreground=[('', calc_font_color(self.bg))])
            s.configure(button_style, padding=0, borderwidth=0, focuscolor=self.bg)

            # add to styles
            Button3d.styles.append(button_style)

        self['style'] = button_style

    def create_images(self):

        shadow_img = create_pil_image(b64=btn_base)
        img = create_pil_image(b64=btn_face, color=self.bg)

        # merge face with base image
        img = mix_images(shadow_img, img)

        pressed_img = img.rotate(180)

        self.img = ImageTk.PhotoImage(img)
        self.pressed_img = ImageTk.PhotoImage(pressed_img) 


all = ['Button3d']


# base6 images

# round square
btn_face = b'iVBORw0KGgoAAAANSUhEUgAAADAAAAAvCAYAAAClgknJAAAACXBIWXMAAA/2AAAP9gGWXJ1PAAAA\nGXRFWHRTb2Z0d2FyZQB3d3cuaW5rc2NhcGUub3Jnm+48GgAAAtZJREFUaIHtmk9rY1UYh5/3vL2T\nzNykIs7tTAX/DNkMlBShIkKpduGmOzdF/AZ+Nb9BN0IVorMJrS12FXDMZoKxC5umTXJ6zuuiie0U\niw5CTwr3gbO4nLP4PYdzz+Z3hH/GraysLBRF4QaDgbtlzZ1Qr9fjYDAI7XY7APHmvNz83tzcrPT7\n/Ue1Wu0d7/2ic64aQtC7ifs6qhpijKMsy05OT0+Pi6I4293dHQP2d+Dr65vN5mKlUvkwxvjMOdcA\nls2sBjy46/BTJiIyAHoxxo6q/joajX47PDw8AQJcCbjV1dXHzrmPVfUzEVk1syfAW0AVSHWMIjAC\n/hSRnpkdxhi/DyG0Dw4O/gCiALK+vl47Pz//AvhKRD4ClrncdU0YfkbkcrfHwCsz2xeRb6vV6net\nVutUt7e3tdfrPVfVb4DPgXeBh0DGlUDKocACUAEWReSpiNQmk8kvGxsbfV1aWqpMJpMvga9F5Mk0\n+M2fex4QYEFEHprZ2yLSvbi4OHDe+8w594mIPDazJLfNm2BmOs36qfc+c8PhUIH3zWwhdbj/ipll\nIvLecDhUBxBjfMR8HpvbEDPLAVwIQbhf4WdICEFSX5H/m1IgNaVAakqB1JQCqSkFUlMKpKYUSE0p\nkJpSIDWlQGpKgdSUAqm5/wKqalzrnO4RpqrmAJxzZ9wvCRORIYDL8zwAXRG5SBzqTfBAN8/z4LIs\n82b2wsz6IhJSJ/s3phn7wE9ZlnlXFIWPMbZEpGVmfS7LtHk8TgaMzex3oGVmPxZF4fXo6IhGozH0\n3p8DuYhUueqFLfGYNZQeOAO6wItpS7m3s7Mzfq0nVtU159ysJ37K/PXEP5vZD2bW3tvbO2baE8/Q\nZrO5WK1WPwghPFPVRoxxGaiTsKkHBsAroBNCeAm83N/fH3CjqZ8hW1tbD7rdbj5PbyVijCfe++Px\neDzsdDoTbnkrcR23tram9Xpd5/21yl+l5WEoEGHVWgAAAABJRU5ErkJggg==\n'


# round square shadow
btn_base = b'iVBORw0KGgoAAAANSUhEUgAAADIAAAAzCAYAAADVY1sUAAAACXBIWXMAAA/2AAAP9gGWXJ1PAAAA\nGXRFWHRTb2Z0d2FyZQB3d3cuaW5rc2NhcGUub3Jnm+48GgAABIZJREFUaIHtmt9v21QUgL9749jp\nmiUd6bIRjYLUAaPtEwIxhJAGCCQQQkJoEkj8Azzw58AbrzyMF3jZHuhLNUBaVLEO1BVpakS7Rkvi\nNd3IEueHfQ8Pdkq2dgXGlpgpn3QV2bqOzufjazs5R7EPIqIBFQ2935whYQABjFJKDpqoBjdERAEW\n4AApwI621d5DHzkG6AFdoA10lVL+/SbvBhhlwQamgCeBPJAlFBqFSEAosANUovEH4O+XHQt2M2ED\nBWABmAeeAXKAHUkOGwO0RaQGrBtjfrUs6yrgEmbqLqzoMwEcB14DzgDPA8eAtIhYxhgdyQ4VEfGD\nILjj+/4NY8xzyWTysDHmJxFxlVJmcK4VBTgJvAi8B7wEHBWRCUJBFTFsDyTkCa318U6nM1Ov1/O1\nWk1SqdT3QHNwrhUFWwDeAk4TZsaOIlcAo5CIUFprEonEhFKq0Gg03q5Wq/aFCxe25+fnL62urnb7\nEy0gSXgpvUx4OTmMZnHvi1KKRCKB4zh6eno6nclkzmxtbTXr9XoLWCG8KaAJAz8FPMXo7lB/i9aa\niYkJVSgU0ul0+p1Wq/UZMEMUrybMyAxwiJhK9NFak8lkyOVyU8D7wMeEJx9NuEYyImId8B2xQClF\nMpkkk8kopdRR4FPgWUBpQImIHQRB7EXgLxmllAZmgQ+IRACUMSYxuvD+HVrvPp9t4F3A0gDGGGWM\nGeXL4YOigDngkAYIgmAkT+6HxCRwZFdkxMH8FxSQ/j8LDJJ4XERG+uvvoTIWiRtjkbgxFokbY5G4\nMRaJG2ORuDEWiRtjkbjx+ImIHFiiixX7xarD/RIMP5wHQ0Tw/b2lRAswItI0xuw9KoYEQUCz2eSe\neI0GfKDc7XYl7peXiNBqtahWq/dmxdNAB/jNdd2g19tTY4wNIoLneWxubrKxscFArB2groFuEARX\nS6XSdr1ep9frxW7hG2PodrtUKhWKxSJbW1v9jAjwO3DHAvx2u71ZKpV+TCaTHy4sLKhsNott24P/\neo8EEcEYQ6fTwXVdlpeXWVpaYnt7u79GAmAR8C2llFy7du3W7du3v15aWnrd87zp2dlZlcvlcBxn\nZDIiQhAEdDodKpUKa2trLC4ucuXKFTzPgzAbVeA7wFgAJ0+e7DqOc7FYLH7ruu4nc3Nz6ZmZGbLZ\nbL+oMnSRfiZ2dnZYX1/n8uXLrKysUKvVMMYI0IgkfgbCcptSSs6dO7d9/vz5L8rlcr5UKr2Zz+fT\nuVxOpVKpkWQlCAJarRau61Iul7l+/To3b94kurs2gIvAV5HQ3cXPs2fP2svLy6fb7fbntm2/Ojk5\necyyLD1Ycx8W/QXeaDRoNpvieZ70ej0jIlWgCHwZyXT2iACcOHFiol6vz4vIR1rrN5RSBWAq6oQY\nukyv1xNjTNsYcwu4AfwAfAP8wkD3w/0CsxzHedr3/VMi8oqIvEDYHZE84JhHgoj0gBqwBlwCVoEN\nwvanXQ4KShPWsPPAEeAwkCYsZw8LA9yJxg5hZ1Az2n8X/+Ts9rvp+s0Fw8yIELY09Tvp+mMPfwK1\nehDEAyyvQAAAAABJRU5ErkJggg==\n'


# progressbar 3d base
progressbar_3d_base = b'iVBORw0KGgoAAAANSUhEUgAAAGQAAABkCAYAAABw4pVUAAAACXBIWXMAABGLAAARiwGaKO0CAAAA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\nSUVORK5CYII=\n'



#!/usr/bin/env python
"""
    AwesomeTkinter, a new tkinter widgets design using custom styles and images

    :copyright: (c) 2020 by Mahmoud Elshahat.

"""

import tkinter as tk

from .button import Button3d
from .frame import Frame3d, ScrollableFrame
from .menu import RightClickMenu
from .progressbar import RadialProgressbar, RadialProgressbar3d
from .scrollbar import SimpleScrollbar
from .text import ScrolledText
from .utils import *
from .config import *


def main():
    
    root = tk.Tk()
    root.config(background=DEFAULT_COLOR)

    f1 = Frame3d(root)
    f1.pack(side='left', expand=True, fill='both', padx=3, pady=3)

    bar = RadialProgressbar3d(f1, fg='cyan', size=120)
    bar.pack(padx=20, pady=20)
    bar.start()

    Button3d(f1, text='3D Button').pack(pady=10)

    f2 = Frame3d(root)
    f2.pack(side='left', expand=True, fill='both', padx=3, pady=3)

    bar = RadialProgressbar(f2, fg='green')
    bar.pack(padx=30, pady=30)
    bar.start()

    btn = Button3d(f2, text='Pressed Button')
    btn.pack(pady=10)

    root.mainloop()















 



import base64
import platform
from tkinter import ttk
import PIL
from PIL import Image, ImageTk, ImageColor, ImageDraw, ImageFilter
import hashlib
import io


def identify_operating_system():
    """identify current operating system

    Returns:
        (str): 'Windows', 'Linux', or 'Darwin' for mac
      """

    return platform.system()


def calc_md5(binary_data):
    return hashlib.md5(binary_data).hexdigest()


def generate_unique_name(*args):
    """get md5 encoding for any arguments that have a string representation

    Returns:
        md5 string
    """
    name = ''.join([str(x) for x in args])

    try:
        name = calc_md5(name.encode())
    except:
        pass

    return name


def invert_color(color):
    """return inverted hex color
    """
    color = color_to_rgba(color)
    r, g, b, a = color

    inverted_color = rgb2hex(255 - r, 255 - g, 255 - b)
    return inverted_color


def rgb2hex(r, g, b):
    return '#{:02x}{:02x}{:02x}'.format(r, g, b)


def change_img_color(img, new_color, old_color=None):
    """Change image color

    Args:
        img: pillow image
        new_color (str): new image color, ex: 'red', '#ff00ff', (255, 0, 0), (255, 0, 0, 255)
        old_color (str): color to be replaced, if omitted, all colors will be replaced with new color keeping
                         alpha channel.

    Returns:
        pillow image
    """

    # convert image to RGBA color scheme
    img = img.convert('RGBA')

    # load pixels data
    pixdata = img.load()

    # handle color
    new_color = color_to_rgba(new_color)
    old_color = color_to_rgba(old_color)

    for y in range(img.size[1]):
        for x in range(img.size[0]):
            alpha = pixdata[x, y][-1]
            if old_color:
                if pixdata[x, y] == old_color:
                    r, g, b, _ = new_color
                    pixdata[x, y] = (r, g, b, alpha)
            else:
                r, g, b, _ = new_color
                pixdata[x, y] = (r, g, b, alpha)

    return img


def resize_img(img, size, keep_aspect_ratio=True):
    """resize image using pillow

    Args:
        img (PIL.Image): pillow image object
        size(int or tuple(in, int)): width of image or tuple of (width, height)
        keep_aspect_ratio(bool): maintain aspect ratio relative to width

    Returns:
        (PIL.Image): pillow image
    """

    if isinstance(size, int):
        size = (size, size)

    # get ratio
    width, height = img.size
    requested_width = size[0]

    if keep_aspect_ratio:
        ratio = width / requested_width
        requested_height = height / ratio
    else:
        requested_height = size[1]

    size = (int(requested_width), int(requested_height))

    img = img.resize(size, resample=PIL.Image.LANCZOS)

    return img


def mix_images(background_img, foreground_img):
    """paste an image on top of another image"""
    background_img = background_img.convert('RGBA')
    foreground_img = foreground_img.convert('RGBA')

    img_w, img_h = foreground_img.size
    bg_w, bg_h = background_img.size
    offset = ((bg_w - img_w) // 2, (bg_h - img_h) // 2)
    background_img.paste(foreground_img, offset, mask=foreground_img)

    return background_img


def color_to_rgba(color):
    """Convert color names or hex notation to RGBA,

    Args:
        color (str): color e.g. 'white' or '#333' or formats like #rgb or #rrggbb

    Returns:
        (4-tuple): tuple of format (r, g, b, a) e.g. it will return (255, 0, 0, 255) for solid red
    """

    if color is None:
        return None

    if isinstance(color, (tuple, list)):
        if len(color) == 3:
            r, g, b = color
            color = (r, g, b, 255)
        return color
    else:
        return ImageColor.getcolor(color, 'RGBA')


def is_dark(color):
    """rough check if color is dark or light

    Returns:
        (bool): True if color is dark, False if light
    """
    r, g, b, a = color_to_rgba(color)

    # calculate lumina, reference https://stackoverflow.com/a/1855903
    lumina = (0.299 * r + 0.587 * g + 0.114 * b) / 255

    return True if lumina < 0.6 else False


def calc_font_color(bg):
    """calculate font color based on given background

    Args:
        bg (str): background color

    Returns:
        (str): color name, e.g. "white" for dark background and "black" for light background
    """

    return 'white' if is_dark(bg) else 'black'


def calc_contrast_color(color, offset):
    """calculate a contrast color

    for darker colors will get a slightly lighter color depend on "offset" and for light colors will get a darker color

    Args:
        color (str): color
        offset (int): 1 to 254

    Returns:
        (str): color
    """

    r, g, b, a = color_to_rgba(color)
    if is_dark(color):
        new_color = [x + offset if x + offset <= 255 else 255 for x in (r, g, b)]
    else:
        new_color = [x - offset if x - offset >= 0 else 0 for x in (r, g, b)]

    return rgb2hex(*new_color)


def text_to_image(text, text_color, bg_color, size):
    """Not implemented"""
    pass
    # img = Image.new('RGBA', size, color_to_rgba(text_color))
    # draw = ImageDraw.Draw(img)
    # font = ImageFont.truetype(current_path + "s.ttf", size - int(0.15 * width))
    # draw.text((pad, -pad), str(num), font=font, fill=color_to_rgba(bg_color))


def create_pil_image(fp=None, color=None, size=None, b64=None):
    """create pillow Image object

    Args:
        fp: A filename (string), pathlib.Path object or a file object. The file object must implement read(), seek(),
            and tell() methods, and be opened in binary mode.
        color (str): color in tkinter format, e.g. 'red', '#3300ff', also color can be a tuple or a list of RGB,
                     e.g. (255, 0, 255)
        size (int or 2-tuple(int, int)): an image required size in a (width, height) tuple
        b64 (str): base64 hex representation of an image, if "fp" is given this parameter will be ignored

    Returns:
        pillow image object
    """

    if not fp and b64:
        fp = io.BytesIO(base64.b64decode(b64))

    img = Image.open(fp)

    # change color
    if color:
        img = change_img_color(img, color)

    # resize
    if size:
        if isinstance(size, int):
            size = (size, size)
        img = resize_img(img, size)

    return img


def create_image(fp=None, img=None, color=None, size=None, b64=None):
    """create tkinter PhotoImage object
    it can modify size and color of original image

    Args:
        fp: A filename (string), pathlib.Path object or a file object. The file object must implement read(), seek(),
            and tell() methods, and be opened in binary mode.
        img (pillow image): if exist fp or b64 arguments will be ignored
        color (str): color in tkinter format, e.g. 'red', '#3300ff', also color can be a tuple or a list of RGB,
                    e.g. (255, 0, 255)
        size (int or 2-tuple(int, int)): an image required size in a (width, height) tuple
        b64 (str): base64 hex representation of an image, if "fp" is given this parameter will be ignored

    Returns:
        tkinter PhotoImage object
    """
    # create pillow image
    if not img:
        img = create_pil_image(fp, color, size, b64)

    # create tkinter images using pillow ImageTk
    img = ImageTk.PhotoImage(img)

    return img


def create_circle(size=100, thickness=None, color='black', antialias=4, offset=0):
    """create high quality circle

    the idea to smooth circle line is to draw a bigger size circle and then resize it to the requested size
    inspired from  https://stackoverflow.com/a/34926008

    Args:
        size (tuple or list, or int): outer diameter of the circle or width of bounding box
        thickness (int): outer line thickness in pixels
        color (str): outer line color
        antialias (int): used to enhance outer line quality and make it smoother
        offset (int): correct cut edges of circle outline

    Returns:
        PIL image: a circle on a transparent image
    """

    if isinstance(size, int):
        requested_size = (size, size)
    else:
        requested_size = size

    # calculate thickness to be 2% of circle diameter
    thickness = thickness or max(size[0] * 2 // 100, 2)

    offset = offset or thickness // 2

    # make things bigger
    size = [x * antialias for x in requested_size]
    thickness *= antialias

    # create a transparent image with a big size
    img = Image.new(size=size, mode='RGBA', color='#0000')

    draw = ImageDraw.Draw(img)

    # draw circle with a required color
    draw.ellipse([offset, offset, size[0] - offset, size[1] - offset], outline=color, fill='#0000', width=thickness)
    # draw.ellipse((0, 0, size[0], size[1]), outline=color, fill='#0000', width=thickness)

    img = img.filter(ImageFilter.BLUR)

    # resize image back to the requested size
    img = img.resize(requested_size, Image.LANCZOS)

    # change color again will enhance quality (weird)
    img = change_img_color(img, color)

    return img


def scroll_with_mousewheel(widget, target=None, modifier='Shift', apply_to_children=False):
    """scroll a widget with mouse wheel

    Args:
        widget: tkinter widget
        target: scrollable tkinter widget, in case you need "widget" to catch mousewheel event and make another widget
                to scroll, useful for child widget in a scrollable frame
        modifier (str): Modifier to use with mousewheel to scroll horizontally, default is shift key
        apply_to_children (bool): bind all children

    Examples:
        scroll_with_mousewheel(my_text_widget, target='my_scrollable_frame')

        to make a scrollable canvas:
        for w in my_canvas:
            scroll_with_mousewheel(w, target=my_canvas)
    """

    def _scroll_with_mousewheel(widget):

        target_widget = target if target else widget

        def scroll_vertically(event):
            # scroll vertically  ----------------------------------
            if event.num == 4 or event.delta > 0:
                target_widget.yview_scroll(-1, "unit")

            elif event.num == 5 or event.delta < 0:
                target_widget.yview_scroll(1, "unit")

            return 'break'

        # bind events for vertical scroll ----------------------------------------------
        if hasattr(target_widget, 'yview_scroll'):
            # linux
            widget.bind("<Button-4>", scroll_vertically, add='+')
            widget.bind("<Button-5>", scroll_vertically, add='+')

            # windows and mac
            widget.bind("<MouseWheel>", scroll_vertically, add='+')

        # scroll horizontally ---------------------------------------
        def scroll_horizontally(event):
            # scroll horizontally
            if event.num == 4 or event.delta > 0:
                target_widget.xview_scroll(-1, "unit")

            elif event.num == 5 or event.delta < 0:
                target_widget.xview_scroll(1, "unit")

            return 'break'

        # bind events for horizontal scroll ----------------------------------------------
        if hasattr(target_widget, 'xview_scroll'):
            # linux
            widget.bind(f"<{modifier}-Button-4>", scroll_horizontally, add='+')
            widget.bind(f"<{modifier}-Button-5>", scroll_horizontally, add='+')

            # windows and mac
            widget.bind(f"<{modifier}-MouseWheel>", scroll_horizontally, add='+')

    _scroll_with_mousewheel(widget)

    def handle_children(w):
        for child in w.winfo_children():
            _scroll_with_mousewheel(child)

            # recursive call
            if child.winfo_children():
                handle_children(child)

    if apply_to_children:
        handle_children(widget)


def unbind_mousewheel(widget):
    """unbind mousewheel for a specific widget, e.g. combobox which have mouswheel scroll by default"""

    # linux
    widget.unbind("<Button-4>")
    widget.unbind("<Button-5>")

    # windows and mac
    widget.unbind("<MouseWheel>")


def get_widget_attribute(widget, attr):
    """get an attribute of a widget

    Args:
        widget: tkinter widget "tk or ttk"
        attr (str): attribute or property e.g. 'background'

    Returns:
        attribute value, e.g. '#ffffff' for a background color
    """

    # if it is ttk based will get style applied, it will raise an error if the widget not a ttk
    try:
        style_name = widget.cget('style') or widget.winfo_class()
        s = ttk.Style()
        value = s.lookup(style_name, attr)
        return value
    except:
        pass

    try:
        # if it's a tk widget will use cget
        return widget.cget(attr)
    except:
        pass

    return None


all = ['identify_operating_system', 'calc_md5', 'generate_unique_name', 'invert_color', 'rgb2hex', 'change_img_color',
       'resize_img', 'mix_images', 'color_to_rgba', 'is_dark', 'calc_font_color', 'calc_contrast_color',
       'text_to_image', 'create_pil_image', 'create_image', 'create_circle', 'scroll_with_mousewheel',
       'unbind_mousewheel', 'get_widget_attribute']




#!/usr/bin/env python

"""
    AwesomeTkinter, a new tkinter widgets design using custom styles and images

    :copyright: (c) 2020 by Mahmoud Elshahat.

"""

import os
import setuptools

# get current directory
path = os.path.realpath(os.path.abspath(__file__))
current_directory = os.path.dirname(path)

# get version
version = {}
with open(f"{current_directory}/awesometkinter/version.py") as f:
    exec(f.read(), version)  # then we can use it as: version['__version__']

# get long description from readme
with open(f"{current_directory}/README.md", "r") as fh:
    long_description = fh.read()

try:
    with open(f"{current_directory}/requirements.txt", "r") as fh:
        requirements = fh.readlines()
except:
    requirements = ['pillow']

setuptools.setup(
    name="AwesomeTkinter",
    version=version['__version__'],
    scripts=[],
    author="Mahmoud Elshahat",
    author_email="",
    description="Pretty tkinter widgets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Aboghazala/AwesomeTkinter",
    packages=setuptools.find_packages(),
    keywords="internet download manager youtube downloader pycurl curl youtube-dl PySimpleGUI",
    project_urls={
        'Source': 'https://github.com/Aboghazala/AwesomeTkinter',
        'Tracker': 'https://github.com/Aboghazala/AwesomeTkinter/issues',
        'Releases': 'https://github.com/Aboghazala/AwesomeTkinter/releases',
        'Screenshots': 'https://github.com/Aboghazala/AwesomeTkinter/issues/1'
    },
    install_requires=requirements,
    entry_points={
        # our executable: "exe file on windows for example"
        'console_scripts': [
            'awesometkinter = awesometkinter.__init__:main',
        ]},
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



"""
    AwesomeTkinter, a new tkinter widgets design using custom styles and images

    :copyright: (c) 2020 by Mahmoud Elshahat.

"""

from .utils import *


class SimpleScrollbar(ttk.Scrollbar):
    """Scrollbar without arrows"""
    style = []

    def __init__(self, parent, orient='horizontal', bg=None, slider_color=None, width=None, **options):
        """intialize scrollbar

        Args:
            orient (srt): 'horizontal' or 'vertical'
            bg (str): trough color
            slider_color (str): slider color
            width (int): slider width
        """

        # initialize super class
        ttk.Scrollbar.__init__(self, parent, **options)

        s = ttk.Style()
        self.bg = bg or 'white'
        self.slider_color = slider_color or 'blue'
        self.width = width or 5

        if orient == 'horizontal':
            # h_scrollbar
            custom_style = f'sb_{len(SimpleScrollbar.style)}.Horizontal.TScrollbar'
            s.layout(custom_style, [('Horizontal.Scrollbar.trough', {'sticky': 'we', 'children':
                                   [('Horizontal.Scrollbar.thumb', {'expand': '1', 'sticky': 'nswe'})]})])
        else:
            # v_scrollbar
            custom_style = f'sb_{len(SimpleScrollbar.style)}.Vertical.TScrollbar'
            s.layout(custom_style, [('Vertical.Scrollbar.trough', {'sticky': 'ns', 'children':
                                   [('Vertical.Scrollbar.thumb', {'expand': '1', 'sticky': 'nswe'})]})])

        SimpleScrollbar.style.append(custom_style)

        s.configure(custom_style, troughcolor=self.bg, borderwidth=1, relief='flat', width=self.width)
        s.map(custom_style, background=[('', self.slider_color)])  # slider color

        self.config(orient=orient, style=custom_style) 
